@@ -25,6 +25,7 @@
 #include "gdesktop.h"
 #include "../vfs/vfs.h"
 
+
 enum {
 	SORT_NAME,
 	SORT_EXTENSION,
@@ -103,13 +104,22 @@ void
 gnome_about_cmd (void)
 {
         GtkWidget *about;
+	static int translated;
         const gchar *authors[] = {
-		"The Midnight Commander Team",
+		N_("The Midnight Commander Team"),
 		"http://www.gnome.org/mc/",
-		"bug reports: mc-bugs@nuclecu.unam.mx",
+		N_("bug reports: http://bugs.gnome.org, or use gnome-bug"),
 		NULL
 	};
 
+	if (!translated){
+		int i;
+		
+		for (i = 0; authors [i]; i++)
+			authors [i] = _(authors [i]);
+		translated = TRUE;
+	}
+	
         about = gnome_about_new (_("GNU Midnight Commander"), VERSION,
 				 "Copyright 1994-1999 the Free Software Foundation",
 				 authors,
@@ -118,22 +128,6 @@ gnome_about_cmd (void)
         gtk_widget_show (about);
 }
 
-void
-gnome_quit_cmd (void)
-{
-	int q = 0;
-
-	if (!confirm_exit)
-		q = 1;
-	else if (query_dialog (_(" The Midnight Commander "),
-			       _(" Do you really want to quit the Midnight Commander? "),
-			       0, 2, _("&Yes"), _("&No")) == 0)
-		q = 1;
-	
-	if (q == 1)
-		gtk_main_quit ();
-}
-
 void
 gnome_open_panel (GtkWidget *widget, WPanel *panel)
 {
@@ -154,16 +148,16 @@ gnome_close_panel (GtkWidget *widget, WPanel *panel)
 	remove_widget (h, panel->filter_w);
 	remove_widget (h, panel);
 
-	/* Kill them */
+	/* Free our own internal stuff */
+	g_free (panel->view_menu_items);
+	g_free (panel->view_toolbar_items);
+
+	/* Kill the widgets */
 	destroy_widget (panel->current_dir);
 	destroy_widget (panel->filter_w);
 	destroy_widget ((Widget *)panel);
 
 	layout_panel_gone (panel);
-
-	g_free (panel->view_menu_items);
-	g_free (panel->view_toolbar_items);
-
 	mc_chdir ("/");
 }
 
@@ -200,7 +194,7 @@ set_view_type (GtkWidget *widget, WPanel *panel, enum list_types type)
 
 	/* Synchronize the widgets */
 
-	for (i = 0; ; i++)
+	for (i = 0; panel->view_menu_items[i] && panel->view_toolbar_items[i]; i++)
 		if (widget == panel->view_menu_items[i]) {
 			gtk_signal_handler_block_by_data (
 				GTK_OBJECT (panel->view_toolbar_items[i]), panel);
@@ -274,7 +268,7 @@ gnome_sort_cmd (GtkWidget *widget, WPanel *panel)
 	cbox1 = gtk_check_button_new_with_label (_("Ignore case sensitivity."));
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (sort_box)->vbox), hbox, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new ("Sort files by "), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("Sort files by ")), FALSE, FALSE, 0);
 
 	omenu = gtk_option_menu_new ();
         gtk_box_pack_start (GTK_BOX (hbox), omenu, FALSE, FALSE, 0);
@@ -619,6 +613,12 @@ gnome_select_all_cmd (GtkWidget *widget, WPanel *panel)
 	do_refresh ();
 }
 
+void
+gnome_start_search (GtkWidget *widget, WPanel *panel)
+{
+	start_search (panel);
+}
+
 void
 gnome_reverse_selection_cmd_panel (WPanel *panel)
 {
@@ -671,7 +671,7 @@ gnome_filter_cmd (GtkWidget *widget, WPanel *panel)
 		gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry))), text3);
 
 	if (easy_patterns)
-		label = gtk_label_new (_("Enter a filter here for files in the panel view.\n\nFor example:\n*.gif will show just gif images"));
+		label = gtk_label_new (_("Enter a filter here for files in the panel view.\n\nFor example:\n*.png will show just png images"));
 	else
 		label = gtk_label_new (_("Enter a Regular Expression to filter files in the panel view."));
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
@@ -807,3 +807,105 @@ gnome_new_launcher (GtkWidget *widget, WPanel *panel)
 
 
 }
+void
+gnome_select (GtkWidget *widget, WPanel *panel)
+{
+    char *reg_exp, *reg_exp_t;
+    int i;
+    int c;
+    int dirflag = 0;
+    GtkWidget *select_dialog;
+    GtkWidget *entry;
+    GtkWidget *label;
+    
+    select_dialog = gnome_dialog_new (_("Select File"), GNOME_STOCK_BUTTON_OK, 
+				      GNOME_STOCK_BUTTON_CANCEL, NULL);
+    gtk_window_set_position (GTK_WINDOW (select_dialog), GTK_WIN_POS_MOUSE);
+    entry = gnome_entry_new ("mc_select");
+    gtk_entry_set_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry))), easy_patterns ? "*" : ".");
+    gnome_entry_load_history (GNOME_ENTRY (entry));
+
+    if (easy_patterns)
+	    label = gtk_label_new (_("Enter a filter here to select files in the panel view with.\n\nFor example:\n*.png will select all png images"));
+    else
+	    label = gtk_label_new (_("Enter a regular expression here to select files in the panel view with."));
+    
+    gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+    gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+    
+    gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+    gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (select_dialog)->vbox), label, FALSE, FALSE, 0);
+    gtk_box_pack_start (GTK_BOX (GNOME_DIALOG (select_dialog)->vbox), entry, FALSE, FALSE, 0);
+    gtk_widget_show_all (GNOME_DIALOG (select_dialog)->vbox);
+    reg_exp = NULL;
+    if (gnome_dialog_run (GNOME_DIALOG (select_dialog)) == 0) {
+	    gtk_widget_hide (select_dialog);
+	    reg_exp = g_strdup (gtk_entry_get_text (GTK_ENTRY (gnome_entry_gtk_entry (GNOME_ENTRY (entry)))));
+    }
+    gtk_widget_destroy (select_dialog);
+    if ((reg_exp == NULL) || (*reg_exp == '\000')) {
+	    g_free (reg_exp);
+	    return;
+    }
+    reg_exp_t = reg_exp;
+
+    /* Check if they specified a directory */
+    if (*reg_exp_t == PATH_SEP){
+        dirflag = 1;
+        reg_exp_t++;
+    }
+    if (reg_exp_t [strlen(reg_exp_t) - 1] == PATH_SEP){
+        dirflag = 1;
+        reg_exp_t [strlen(reg_exp_t) - 1] = 0;
+    }
+
+    for (i = 0; i < panel->count; i++){
+        if (!strcmp (panel->dir.list [i].fname, ".."))
+            continue;
+	if (S_ISDIR (panel->dir.list [i].buf.st_mode)){
+	    if (!dirflag)
+                continue;
+        } else {
+            if (dirflag)
+                continue;
+	}
+	c = regexp_match (reg_exp_t, panel->dir.list [i].fname, match_file);
+	if (c == -1){
+	    message (1, MSG_ERROR, _("  Malformed regular expression  "));
+	    g_free (reg_exp);
+	    return;
+	}
+	if (c){
+	    do_file_mark (panel, i, 1);
+	}
+    }
+    paint_panel (panel);
+    g_free (reg_exp);
+}
+void
+set_cursor_busy (WPanel *panel)
+{
+	GdkCursor *cursor;
+
+	if (is_a_desktop_panel (panel))
+		return;
+
+	cursor = gdk_cursor_new (GDK_WATCH);
+	gdk_window_set_cursor (GTK_WIDGET (panel->xwindow)->window, cursor);
+	gdk_cursor_destroy (cursor);
+	gdk_flush ();
+
+}
+void
+set_cursor_normal (WPanel *panel)
+{
+	GdkCursor *cursor;
+
+	if (is_a_desktop_panel (panel))
+		return;
+	
+	cursor = gdk_cursor_new (GDK_TOP_LEFT_ARROW);
+	gdk_window_set_cursor (GTK_WIDGET (panel->xwindow)->window, cursor);
+	gdk_cursor_destroy (cursor);
+	gdk_flush ();
+}
