@@ -6,7 +6,6 @@ void gnome_compare_panels     	  (void);
 void gnome_open_terminal      	  (void);
 void gnome_open_terminal_with_cmd (const char *command);
 void gnome_about_cmd              (void);
-void gnome_quit_cmd               (void);
 void gnome_open_panel             (GtkWidget *widget, WPanel *panel);
 void gnome_close_panel            (GtkWidget *widget, WPanel *panel);
 
@@ -17,6 +16,7 @@ void gnome_custom_view_cmd        (GtkWidget *widget, WPanel *panel);
 
 void gnome_sort_cmd               (GtkWidget *widget, WPanel *panel);
 void gnome_select_all_cmd         (GtkWidget *widget, WPanel *panel);
+void gnome_start_search		  (GtkWidget *widget, WPanel *panel);
 void gnome_filter_cmd             (GtkWidget *widget, WPanel *panel);
 void gnome_external_panelize      (GtkWidget *widget, WPanel *panel);
 void gnome_open_files             (GtkWidget *widget, WPanel *panel);
@@ -24,5 +24,8 @@ void gnome_run_new                (GtkWidget *widget, GnomeDesktopEntry *gde);
 void gnome_mkdir_cmd              (GtkWidget *widget, WPanel *panel);
 void gnome_new_launcher           (GtkWidget *widget, WPanel *panel);
 void gnome_reverse_selection_cmd_panel (WPanel *panel);
+void gnome_select                 (GtkWidget *widget, WPanel *panel);
+void set_cursor_normal		  (WPanel *panel);
+void set_cursor_busy		  (WPanel *panel);
 
 #endif /* __GCMD_H */
