@@ -134,8 +134,13 @@ static DesktopIconInfo *desktop_icon_info_new (char *filename, char *url, char *
 static void
 get_slot_from_pos (int x, int y, int *u, int *v)
 {
-	*u = (x + DESKTOP_SNAP_X / 2) / DESKTOP_SNAP_X;
-	*v = (y + DESKTOP_SNAP_Y / 2) / DESKTOP_SNAP_Y;
+	int uu, vv;
+
+	uu = (x + DESKTOP_SNAP_X / 2) / DESKTOP_SNAP_X;
+	vv = (y + DESKTOP_SNAP_Y / 2) / DESKTOP_SNAP_Y;
+
+	*u = CLAMP (uu, 0, layout_cols - 1);
+	*v = CLAMP (vv, 0, layout_rows - 1);
 }
 
 /* Looks for a free slot in the layout_slots array and returns the coordinates
@@ -298,6 +303,9 @@ desktop_icon_info_place (DesktopIconInfo *dii, int xpos, int ypos)
 static void
 desktop_icon_info_destroy (DesktopIconInfo *dii)
 {
+	if (last_selected_icon == dii)
+		last_selected_icon = NULL;
+
 	gtk_widget_destroy (dii->dicon);
 	remove_from_slot (dii);
 
@@ -1097,6 +1105,134 @@ try_to_mount (char *filename, file_entry *fe)
 	return do_mount_umount (filename, TRUE);
 }
 
+/* This is a HORRIBLE HACK.  It creates a temporary panel structure for gpopup's
+ * perusal.  Once gmc is rewritten, all file lists including panels will be a
+ * single data structure, and the world will be happy again.
+ */
+static WPanel *
+create_panel_from_desktop (void)
+{
+	WPanel *panel;
+	int nicons, count;
+	int marked_count, dir_marked_count;
+	long total;
+	int selected_index;
+	int i;
+	file_entry *fe;
+	GList *l;
+	struct stat s;
+
+	panel = g_new0 (WPanel, 1);
+
+	/* Count the number of desktop icons */
+
+	nicons = 0;
+	for (i = 0; i < layout_cols * layout_rows; i++)
+		nicons += layout_slots[i].num_icons;
+
+	/* Create the file entry list */
+
+	panel->dir.size = nicons;
+
+	count = 0;
+	marked_count = 0;
+	dir_marked_count = 0;
+	total = 0;
+	selected_index = -1;
+
+	if (nicons != 0) {
+		panel->dir.list = g_new (file_entry, nicons);
+
+		fe = panel->dir.list;
+
+		for (i = 0; i < layout_cols * layout_rows; i++)
+			for (l = layout_slots[i].icons; l; l = l->next) {
+				DesktopIconInfo *dii;
+				char *full_name;
+
+				dii = l->data;
+				full_name = g_concat_dir_and_file (desktop_directory, dii->filename);
+				if (mc_lstat (full_name, &s) == -1) {
+					g_warning ("Could not stat %s, bad things will happen",
+						   full_name);
+					continue;
+				}
+
+				file_entry_fill (fe, &s, full_name);
+				if (dii->selected) {
+					marked_count++;
+					fe->f.marked = TRUE;
+
+					if (S_ISDIR (fe->buf.st_mode)) {
+						dir_marked_count++;
+						if (fe->f.dir_size_computed)
+							total += fe->buf.st_size;
+					} else
+						total += fe->buf.st_size;
+				}
+
+				g_free (full_name);
+				fe++;
+				count++;
+			}
+	}
+
+	/* Fill the rest of the panel structure */
+
+	panel->list_type = list_icons;
+	strncpy (panel->cwd, desktop_directory, sizeof (panel->cwd));
+	panel->count = count; /* the actual number of lstat()ed files */
+	panel->marked = marked_count;
+	panel->dirs_marked = dir_marked_count;
+	panel->total = total;
+	panel->selected = selected_index;
+	panel->is_a_desktop_panel = TRUE;
+	panel->id = -1;
+
+	return panel;
+}
+
+/* Pushes our hacked-up desktop panel into the containers list */
+WPanel *
+push_desktop_panel_hack (void)
+{
+	WPanel *panel;
+	PanelContainer *container;
+	
+	panel = create_panel_from_desktop ();
+	container = g_new (PanelContainer, 1);
+	container->splitted = FALSE;
+	container->panel = panel;
+
+	containers = g_list_append (containers, container);
+
+	if (!current_panel_ptr)
+		current_panel_ptr = container;
+	else if (!other_panel_ptr)
+		other_panel_ptr = container;
+
+	/* Set it as the current panel and invoke the menu */
+
+	set_current_panel (panel);
+	mc_chdir (desktop_directory);
+	return panel;
+}
+
+/* Frees our hacked-up panel created in the function above */
+static void
+free_panel_from_desktop (WPanel *panel)
+{
+	int i;
+
+	for (i = 0; i < panel->count; i++)
+		g_free (panel->dir.list[i].fname);
+	
+	if (panel->dir.list)
+		g_free (panel->dir.list);
+
+	g_free (panel);
+}
+
 /**
  * desktop_icon_info_open:
  * @dii: The desktop icon to open.
@@ -1147,39 +1283,15 @@ desktop_icon_info_open (DesktopIconInfo *dii)
 			new_panel_at (point);
 		g_free (point);
 	} else {
+		WPanel *panel;
+		panel = push_desktop_panel_hack ();
+		/* we need to special case the new dir.  Otherwise we'd try to cd */
 		if (S_ISDIR (fe->buf.st_mode) || link_isdir (fe))
 			new_panel_at (filename);
-		else {
-			int size;
-			char *buf;
-
-			if (gnome_metadata_get (filename,"fm-open", &size, &buf) == 0){
-				g_free (buf);
-				gmc_open_filename (filename, NULL);
-				goto out;
-			}
-
-			if (gnome_metadata_get (filename, "open", &size, &buf) == 0){
-				g_free (buf);
-				gmc_open_filename (filename, NULL);
-				goto out;
-			}
-
-			if (is_exe (fe->buf.st_mode) && if_link_is_exe (desktop_directory, fe)){
-				int needs_terminal = 0;
-
-				if (gnome_metadata_get (filename, "flags", &size, &buf) == 0){
-					needs_terminal = strstr (buf, "needsterminal") != 0;
-					g_free (buf);
-				}
-
-				if (needs_terminal)
-					gnome_open_terminal_with_cmd (filename);
-				else
-					my_system (EXECUTE_AS_SHELL, shell, filename);
-			} else
-				gmc_open_filename (filename, NULL);
-		}
+		else
+			do_enter_on_file_entry (fe);
+		layout_panel_gone (panel);
+		free_panel_from_desktop (panel);
 	}
 
  out:
@@ -1283,134 +1395,6 @@ desktop_icon_info_get_by_filename (char *filename)
 	return NULL;
 }
 
-/* This is a HORRIBLE HACK.  It creates a temporary panel structure for gpopup's
- * perusal.  Once gmc is rewritten, all file lists including panels will be a
- * single data structure, and the world will be happy again.
- */
-static WPanel *
-create_panel_from_desktop (void)
-{
-	WPanel *panel;
-	int nicons, count;
-	int marked_count, dir_marked_count;
-	long total;
-	int selected_index;
-	int i;
-	file_entry *fe;
-	GList *l;
-	struct stat s;
-
-	panel = g_new0 (WPanel, 1);
-
-	/* Count the number of desktop icons */
-
-	nicons = 0;
-	for (i = 0; i < layout_cols * layout_rows; i++)
-		nicons += layout_slots[i].num_icons;
-
-	/* Create the file entry list */
-
-	panel->dir.size = nicons;
-
-	count = 0;
-	marked_count = 0;
-	dir_marked_count = 0;
-	total = 0;
-	selected_index = -1;
-
-	if (nicons != 0) {
-		panel->dir.list = g_new (file_entry, nicons);
-
-		fe = panel->dir.list;
-
-		for (i = 0; i < layout_cols * layout_rows; i++)
-			for (l = layout_slots[i].icons; l; l = l->next) {
-				DesktopIconInfo *dii;
-				char *full_name;
-
-				dii = l->data;
-				full_name = g_concat_dir_and_file (desktop_directory, dii->filename);
-				if (mc_lstat (full_name, &s) == -1) {
-					g_warning ("Could not stat %s, bad things will happen",
-						   full_name);
-					continue;
-				}
-
-				file_entry_fill (fe, &s, full_name);
-				if (dii->selected) {
-					marked_count++;
-					fe->f.marked = TRUE;
-
-					if (S_ISDIR (fe->buf.st_mode)) {
-						dir_marked_count++;
-						if (fe->f.dir_size_computed)
-							total += fe->buf.st_size;
-					} else
-						total += fe->buf.st_size;
-				}
-
-				g_free (full_name);
-				fe++;
-				count++;
-			}
-	}
-
-	/* Fill the rest of the panel structure */
-
-	panel->list_type = list_icons;
-	strncpy (panel->cwd, desktop_directory, sizeof (panel->cwd));
-	panel->count = count; /* the actual number of lstat()ed files */
-	panel->marked = marked_count;
-	panel->dirs_marked = dir_marked_count;
-	panel->total = total;
-	panel->selected = selected_index;
-	panel->is_a_desktop_panel = TRUE;
-	panel->id = -1;
-
-	return panel;
-}
-
-WPanel *
-push_desktop_panel_hack (void)
-{
-	WPanel *panel;
-	PanelContainer *container;
-	
-	panel = create_panel_from_desktop ();
-	container = g_new (PanelContainer, 1);
-	container->splitted = FALSE;
-	container->panel = panel;
-
-	containers = g_list_append (containers, container);
-
-	if (!current_panel_ptr)
-		current_panel_ptr = container;
-	else if (!other_panel_ptr)
-		other_panel_ptr = container;
-
-	/* Set it as the current panel and invoke the menu */
-
-	set_current_panel (panel);
-	mc_chdir (desktop_directory);
-	return panel;
-}
-
-
-/* Frees our hacked-up panel created in the function above */
-static void
-free_panel_from_desktop (WPanel *panel)
-{
-	int i;
-
-	for (i = 0; i < panel->count; i++)
-		g_free (panel->dir.list[i].fname);
-	
-	if (panel->dir.list)
-		g_free (panel->dir.list);
-
-	g_free (panel);
-}
-
 /* Used to execute the popup menu for desktop icons */
 static void
 do_popup_menu (DesktopIconInfo *dii, GdkEventButton *event)
@@ -1762,10 +1746,10 @@ setup_icon_dnd_source (DesktopIconInfo *dii)
 			    (GtkSignalFunc) drag_begin,
 			    dii);
 	gtk_signal_connect (GTK_OBJECT (DESKTOP_ICON (dii->dicon)->canvas), "drag_data_get",
-			    GTK_SIGNAL_FUNC (drag_data_get),
+			    (GtkSignalFunc) drag_data_get,
 			    dii);
 	gtk_signal_connect (GTK_OBJECT (DESKTOP_ICON (dii->dicon)->canvas), "drag_end",
-			    GTK_SIGNAL_FUNC (drag_end),
+			    (GtkSignalFunc) drag_end,
 			    dii);
 }
 
@@ -2680,6 +2664,7 @@ click_proxy_button_release (GtkWidget *widget, GdkEventButton *event, gpointer d
 	update_drag_selection (event->x, event->y);
 
 	XUngrabServer (GDK_DISPLAY ());
+	gdk_flush ();
 
 	return TRUE;
 }
