@@ -133,7 +133,7 @@ set_current_panel (WPanel *panel)
 
 	if (g_list_length (containers) > 1)
 		other_panel_ptr = current_panel_ptr;
-	
+
 	for (p = containers; p; p = p->next){
 		if (((PanelContainer *)p->data)->panel == panel){
 			current_panel_ptr = p->data;
@@ -294,7 +294,6 @@ save_panel_types (void)
 			panel_save_setup (pc->panel, pc->panel->panel_name);
 	}
 }
-
 static void
 run_cmd (void)
 {
@@ -395,9 +394,12 @@ GnomeUIInfo gnome_panel_file_menu [] = {
 GnomeUIInfo gnome_panel_edit_menu [] = {
 	{ GNOME_APP_UI_ITEM,  N_("Select _All"),        N_("Select all files in the current Panel"), gnome_select_all_cmd,
 	  NULL, NULL, 0, NULL, 'a', GDK_CONTROL_MASK  },
-	GNOMEUIINFO_ITEM_NONE(N_("_Select Files..."),  N_("Select a group of files"), select_cmd),
+	GNOMEUIINFO_ITEM_NONE(N_("_Select Files..."),  N_("Select a group of files"), gnome_select),
 	GNOMEUIINFO_ITEM_NONE(N_("_Invert Selection"), N_("Reverses the list of tagged files"), reverse_selection_cmd),
 	GNOMEUIINFO_SEPARATOR,
+	{ GNOME_APP_UI_ITEM, N_("Search"),		N_("Search for a file in the current Panel"), gnome_start_search,
+	  NULL, NULL, 0, NULL, 's', GDK_CONTROL_MASK  },
+	GNOMEUIINFO_SEPARATOR,
         GNOMEUIINFO_ITEM_NONE(N_("_Rescan Directory"), N_("Rescan the directory contents"), reread_cmd),
 	GNOMEUIINFO_SEPARATOR,
 	GNOMEUIINFO_MENU_PREFERENCES_ITEM(gnome_configure_box, NULL),
@@ -675,19 +677,20 @@ my_app_create_menus (GnomeApp *app, GnomeUIInfo *uiinfo, void *data)
 /**
  * copy_uiinfo_widgets:
  * @uiinfo: A GnomeUIInfo array
- * @dest: The destination array will be placed here
  * 
  * Allocates an array of widgets and copies the widgets from the uiinfo array to
  * it.  The array will be NULL-terminated.
+ *
+ * Returns: The allocated array of widgets.
  **/
-void
-copy_uiinfo_widgets (GnomeUIInfo *uiinfo, gpointer **dest)
+gpointer *
+copy_uiinfo_widgets (GnomeUIInfo *uiinfo)
 {
+	gpointer *dest;
 	int n;
 	int i;
 
-	g_return_if_fail (uiinfo != NULL);
-	g_return_if_fail (dest != NULL);
+	g_return_val_if_fail (uiinfo != NULL, NULL);
 
 	/* Count number of items */
 
@@ -695,12 +698,14 @@ copy_uiinfo_widgets (GnomeUIInfo *uiinfo, gpointer **dest)
 
 	/* Copy the widgets */
 
-	*dest = g_new (gpointer, n + 1);
+	dest = g_new (gpointer, n + 1);
 
 	for (i = 0; i < n; i++)
-		(*dest)[i] = uiinfo[i].widget;
+		dest[i] = uiinfo[i].widget;
 
-	(*dest)[i] = NULL;
+	dest[i] = NULL;
+
+	return dest;
 }
 
 WPanel *
@@ -722,9 +727,10 @@ create_container (Dlg_head *h, char *name, char *geometry)
 
 	/* Geometry configuration */
 	if (width != -1 && height != -1)
-		gtk_widget_set_usize (GTK_WIDGET (app), width, height);
+		gtk_window_set_default_size (GTK_WINDOW (app), width, height);
 	else
-		gtk_widget_set_usize (GTK_WIDGET (app), 540, 360);
+		gtk_window_set_default_size (GTK_WINDOW (app), 540, 360);
+
 	if (xpos != -1 && ypos != -1)
 		gtk_widget_set_uposition (GTK_WIDGET (app), xpos, ypos);
 
@@ -747,7 +753,7 @@ create_container (Dlg_head *h, char *name, char *geometry)
 		uiinfo = gnome_panel_menu_with_desktop;
 
 	my_app_create_menus (GNOME_APP (app), uiinfo, panel);
-	copy_uiinfo_widgets (panel_view_menu_uiinfo, &panel->view_menu_items);
+	panel->view_menu_items = copy_uiinfo_widgets (panel_view_menu_uiinfo);
 
 	create_new_menu (GNOME_APP (app), panel);
 
@@ -794,8 +800,8 @@ new_panel_with_geometry_at (char *dir, char *geometry)
 
 	mc_chdir (dir);
 	panel = create_container (desktop_dlg, dir, geometry);
-	add_widget (desktop_dlg, panel);
 	set_current_panel (panel);
+	add_widget (desktop_dlg, panel);
 #if 0
 	x_flush_events ();
 #endif
