@@ -26,6 +26,7 @@
 #include "gsession.h"
 #include "command.h"
 #include "cmd.h"
+#include <gdk/gdkx.h>
 
 GdkColorContext *mc_cc;
 
@@ -106,7 +107,6 @@ int
 xtoolkit_init (int *argc, char *argv [])
 {
 	gmc_color_init ();
-
 	return 0;
 }
 
@@ -624,14 +624,14 @@ void
 gnome_check_super_user (void)
 {
 	GtkWidget *warning_dlg;
-	GnomeClient *client;
 
 	if (geteuid () != 0)
 		return;
 
 	warning_dlg = gnome_message_box_new (
-		_("You are running the GNOME Midnight Commander as root. \n\n"
-		  "You will not be protected from severly damaging your system."),
+		_("You are running the GNOME File Manager as root.\n\n"
+		  "As root, you can damage your system if you are not careful, and the "
+		  "GNOME File Manager will not stop you from doing it."),
 		GNOME_MESSAGE_BOX_WARNING,
 		GNOME_STOCK_BUTTON_OK, NULL);
 
