@@ -73,7 +73,7 @@
 
 #include "x.h"
 
-/* "$Id: layout.c,v 1.17 1999/01/27 01:09:08 timur Exp $" */
+/* "$Id: layout.c,v 1.18 1999/04/06 19:00:32 unammx Exp $" */
 
 /* Controls the display of the rotating dash on the verbose mode */
 int nice_rotating_dash = 1;
@@ -142,7 +142,8 @@ static int _filetype_mode;
 
 static int height;
 
-#define MINWIDTH 10
+/* Width 12 for a wee Quick (Hex) View */
+#define MINWIDTH 12
 #define MINHEIGHT 5
 
 #define BY      12
@@ -631,6 +632,10 @@ panel_do_cols (int index)
 	set_panel_formats ((WPanel *) panels [index].widget);
     else {
 	panel_update_cols (panels [index].widget, frame_half);
+
+	/* In case of an Hex Quick View must review line length */
+	if (get_display_type (index) == view_quick)
+	    view_update_bytes_per_line ((WView *) panels [index].widget);
     }
 }
 
