@@ -144,7 +144,7 @@
 #include "gsession.h"
 #endif
 
-/* "$Id: main.c,v 1.90 1999/03/30 06:10:21 unammx Exp $" */
+/* "$Id: main.c,v 1.91 1999/04/06 03:49:43 unammx Exp $" */
 
 /* When the modes are active, left_panel, right_panel and tree_panel */
 /* Point to a proper data structure.  You should check with the functions */
@@ -3114,6 +3114,9 @@ int main (int argc, char *argv [])
 
     /* Program main loop */
     do_nc ();
+
+    /* Save the tree store */
+    mc_tree_store_save ();
     
     /* Virtual File System shutdown */
     vfs_shut ();
