@@ -9,13 +9,15 @@ int check_format_cd (char *);
 char *check_patterns (char*);
 
 #ifdef OS2_NT
-#   define MC_LOCAL_MENU  "mc.men"
-#   define MC_GLOBAL_MENU "mc.men"
-#   define MC_HOME_MENU   "mc.men"
+#   define MC_LOCAL_MENU  "mc.mnu"
+#   define MC_GLOBAL_MENU "mc.mnu"
+#   define MC_HOME_MENU   "mc.mnu"
+#   define MC_HINT        "mc.hnt"
 #else
 #   define MC_GLOBAL_MENU "mc.menu"
 #   define MC_LOCAL_MENU  ".mc.menu"
 #   define MC_HOME_MENU   ".mc/menu"
+#   define MC_HINT        "mc.hint"
 #endif
 
 #endif
